import time
from PyQt5.QtWidgets import QWidget, QLabel, QPushButton, QCalendarWidget
from PyQt5.QtCore import QTime, Qt, pyqtSignal
from PyQt5.QtGui import QTextCharFormat, QColor

from window_default import WindowDefaultWidget

BASE_YEAR = 2019


class PrefDateWidget(WindowDefaultWidget):

    shown = pyqtSignal()

    def __init__(self, protocol, parent=None):
        super().__init__(parent)

        self.parent = parent
        self.protocol = protocol

        self.month_selector = None
        self.year_selector = None

        self.months = [
            _("Январь"), _("Февраль"), _("Март"), _("Апрель"),
            _("Май"), _("Июнь"), _("Июль"), _("Август"),
            _("Сентябрь"), _("Октябрь"), _("Ноябрь"), _("Декабрь"),
        ]

        self.page = { 'year': BASE_YEAR, 'month' : 0}

        self.makeDate()
        self.makeMonthSelector()
        self.makeYearSelector()

        save = QPushButton(_("Сохранить"), self)
        save.setFlat(True)
        save.setGeometry(283, 418, 515-283, 470-418)
        save.setObjectName('save')
        save.clicked.connect(self.saveClick)
        save.clicked.connect(self.go_back.emit)

        self.shown.connect(self.calendar.showToday)

    # # #

    def makeMonthSelector(self):
        widget = QWidget(self)
        widget.setGeometry(274, 59, 617-274, 398-59)
        widget.setObjectName('selector')
        widget.hide()

        up = QPushButton(None, widget)
        up.setGeometry(222, 15, 106, 64)
        up.setObjectName('up')
        up.clicked.connect(lambda: self.monthUpdate(-1))

        down = QPushButton(None, widget)
        down.setGeometry(222, 261, 106, 64)
        down.setObjectName('down')
        down.clicked.connect(lambda: self.monthUpdate(+1))
        widget.buttons = []
        widget.offset = 0

        for i in range(0, 6):
            button = QPushButton(None, widget)
            button.setGeometry(36, 15 + i * 52, 186, 52)
            button.setObjectName('item')
            button.clicked.connect(
                (lambda n: lambda: self.monthButtonClick(n))(i))
            widget.buttons.append(button)

        self.month_selector = widget
        self.monthUpdate()

    def monthUpdate(self, delta=0):
        month_selector = self.month_selector
        month_selector.offset += delta
        if month_selector.offset < 0:
            month_selector.offset = 11
        elif month_selector.offset > 11:
            month_selector.offset = 0

        for i in range(0, 6):
            button = month_selector.buttons[i]
            button.setText(self.months[(i + month_selector.offset) % 12])

    def monthButtonClick(self, n):
        print('monthButtonClick', n)
        offset = self.month_selector.offset
        # date = self.calendar.selectedDate()
        self.page['month'] = (n + offset) % 12
        self.updateCurrent()

    # # #

    def makeYearSelector(self):
        widget = QWidget(self)
        widget.setGeometry(385, 59, 657-385, 398-59)
        widget.setObjectName('selector')
        widget.hide()

        up = QPushButton(None, widget)
        up.setGeometry(151, 15, 106, 64)
        up.setObjectName('up')
        up.clicked.connect(lambda: self.yearUpdate(-1))

        down = QPushButton(None, widget)
        down.setGeometry(151, 261, 106, 64)
        down.setObjectName('down')
        down.clicked.connect(lambda: self.yearUpdate(+1))
        widget.buttons = []
        widget.offset = 0

        for i in range(0, 6):
            button = QPushButton(None, widget)
            button.setGeometry(37, 15 + i * 52, 114, 52)
            button.setObjectName('item')
            button.clicked.connect(
                (lambda n: lambda: self.yearButtonClick(n))(i))
            widget.buttons.append(button)
        self.year_selector = widget
        self.yearUpdate()

    def yearUpdate(self, delta = 0):
        year_selector = self.year_selector

        year_selector.offset += delta
        if year_selector.offset < 0:
            year_selector.offset = 0
        elif year_selector.offset > 30:
            year_selector.offset = 30

        for i in range(0, 6):
            button = year_selector.buttons[i]
            button.setText(str(BASE_YEAR + i + year_selector.offset))

    def yearButtonClick(self, n):
        offset = self.year_selector.offset
        # date = self.calendar.selectedDate()
        self.page['year'] = n + offset + BASE_YEAR
        self.updateCurrent()

    # # #

    def makeDate(self):
        month = QPushButton(None, self)
        month.setGeometry(283, 14, 410-283, 65-14)
        month.clicked.connect(self.monthClick)
        self.month = month

        year = QPushButton(None, self)
        year.setGeometry(410, 14, 515-410, 65-14)
        year.clicked.connect(self.yearClick)
        self.year = year

        for (n, day) in enumerate([
                _("Пн"), _("Вт"), _("Ср"), _("Чт"), _("Пт"), _("Сб"), _("Вс")]):
            label = QLabel(day, self)
            label.setGeometry(29 + (n * 106), 65, 106, 32)
            label.setObjectName(str(n))

        calendar = QCalendarWidget(self)

        calendar.setGeometry(29, 96, 771-29, 410-96)
        calendar.setVerticalHeaderFormat(QCalendarWidget.NoVerticalHeader)
        calendar.setHorizontalHeaderFormat(QCalendarWidget.NoHorizontalHeader)
        calendar.setNavigationBarVisible(False)
        calendar.setGridVisible(True)
        calendar.currentPageChanged.connect(self.updateCurrent)

        frmt = QTextCharFormat()
        frmt.setForeground(QColor(255, 255, 255))
        frmt.setBackground(QColor(255, 255, 255, 0))

        calendar.setWeekdayTextFormat(Qt.Monday, frmt)
        calendar.setWeekdayTextFormat(Qt.Tuesday, frmt)
        calendar.setWeekdayTextFormat(Qt.Wednesday, frmt)
        calendar.setWeekdayTextFormat(Qt.Thursday, frmt)
        calendar.setWeekdayTextFormat(Qt.Friday, frmt)

        frmt = QTextCharFormat()
        frmt.setForeground(QColor(0x7B, 0x78, 0x78))
        frmt.setBackground(QColor(255, 255, 255, 0))

        calendar.setWeekdayTextFormat(Qt.Saturday, frmt)
        calendar.setWeekdayTextFormat(Qt.Sunday, frmt)

        self.calendar = calendar

        now = calendar.selectedDate()
        self.updateCurrent(now.year(), now.month())

    # Buttons handlers

    def monthClick(self):
        self.year_selector.hide()
        self.month_selector.show()

    def yearClick(self):
        self.month_selector.hide()
        self.year_selector.show()

    def updateCurrent(self, year=None, month=None):
        if year:
            self.page['year'] = year
        if month:
            self.page['month'] = month - 1
        self.month.setText(self.months[self.page['month']])
        self.year.setText(str(self.page['year']))
        self.calendar.setCurrentPage(self.page['year'], self.page['month'] + 1)
        if not self.year_selector:
            return
        self.year_selector.hide()
        self.month_selector.hide()

    def saveClick(self):
        prev = QTime().currentTime()
        date = self.calendar.selectedDate()

        x = (
            date.year(), date.month(), date.day(),
            prev.hour(), prev.minute(), prev.second(),
            0, 0, 0
        )

        self.protocol.setSysTime(time.mktime(x))

    def showEvent(self, event):
        super().showEvent(event)
        self.shown.emit()
