from PyQt5.QtWidgets import \
    QWidget, QLabel, QPushButton, QRadioButton, QButtonGroup
from PyQt5.QtCore import Qt, pyqtSignal

from window_default import WindowDefaultWidget


class PrefNetworkWidget(WindowDefaultWidget):

    accept = pyqtSignal(dict)

    def __init__(self, protocol, parent=None):
        super().__init__(parent)

        self.protocol = protocol
        self.parent = parent

        protocol.answer.connect(self.protocolAnswer)

        # # #

        group1 = QWidget(self)
        group1.setGeometry(28, 75, 350-28, 272-75)
        group1.setObjectName('group1')

        button = QRadioButton(_("Заводские"), group1)
        button.clicked.connect(self.defaultMode)
        button.setGeometry(0, 0, 350-28, 35)
        self.radio_default = button

        button = QRadioButton(_("DHCP-сервер"), group1)
        button.clicked.connect(self.dhcpMode)
        button.setGeometry(0, 69, 350-28, 35)
        self.radio_dhcp = button

        button = QRadioButton(_("Пользовательские"), group1)
        button.clicked.connect(self.userMode)
        button.setGeometry(0, 133, 350-28, 35)
        self.radio_user = button

        # # #

        label = QLabel(
            _("Сетевые настройки устанавливаются DHCP-сервером"), self)
        label.setGeometry(363, 75, 400, 200)
        label.setWordWrap(True)
        label.setObjectName('dhcp_message')
        self.lbl_dhcp = label

        group2 = QWidget(self)
        group2.setGeometry(363, 75, 770-363, 328-75)
        group2.setObjectName('group2')
        self.group2 = group2

        QLabel(_("IP-адрес"), group2).setGeometry(0, 0, 200, 79)
        QLabel(_("Адрес шлюза"), group2).setGeometry(0, 87, 200, 79)
        QLabel(_("Маска подсети"), group2).setGeometry(0, 174, 200, 79)

        group = QButtonGroup(group2)
        self.group = group
        self.current = None

        line = QRadioButton(None, group2)
        line.setGeometry(206, 0, 200, 79)
        line.clicked.connect(self.selectLine)
        line.setProperty('var', 'net_ip')
        group.addButton(line)
        self.net_ip = line

        line = QRadioButton(None, group2)
        line.setGeometry(206, 87, 200, 79)
        line.clicked.connect(self.selectLine)
        line.setProperty('var', 'net_gateway')
        group.addButton(line)
        self.net_gateway = line

        line = QRadioButton(None, group2)
        line.setGeometry(206, 174, 200, 79)
        line.clicked.connect(self.selectLine)
        line.setProperty('var', 'net_mask')
        group.addButton(line)
        self.net_mask = line

        # # #

        labels = [
            {'label': '1', 'name': None},
            {'label': '2', 'name': None},
            {'label': '3', 'name': None},
            {'label': '4', 'name': None},
            {'label': '5', 'name': None},
            {'label': '6', 'name': None},
            {'label': '7', 'name': None},
            {'label': '8', 'name': None},
            {'label': '9', 'name': None},
            {'label': None, 'name': 'delete'},
            {'label': '0', 'name': None},
            {'label': '.', 'name': None}
        ]

        i = 0

        group3 = QWidget(self)
        group3.setGeometry(28, 75, 344-28, 366-75)
        group3.setObjectName('group3')
        group3.hide()
        self.group3 = group3

        for row in range(0, 4):
            for col in range(0, 3):
                label = labels[i]['label']
                button = QPushButton(label, group3)
                button.setGeometry(col * 94, row * 70, 95, 71)

                name = labels[i]['name']

                if name:
                    button.setObjectName(name)

                if i == 9:
                    button.clicked.connect(self.deleteClick)
                    self.delete = button
                else:
                    button.clicked.connect(self.charClick)

                i += 1

        # # #

        protocol.getSettings('net')
        protocol.getSysinfo()

        save = QPushButton(_("Сохранить"), self)
        save.setFlat(True)
        save.setGeometry(28, 382, 310-28, 460-382)
        save.setObjectName('save')
        save.setFocusPolicy(Qt.NoFocus)
        save.clicked.connect(
            lambda: self.accept.emit({
                'net_ip': self.net_ip.text(),
                'net_gateway': self.net_gateway.text(),
                'net_mask': self.net_mask.text(),
                'net_mode': self.mode, }))
        save.clicked.connect(self.go_back.emit)
        self.accept.connect(self.protocol.setSettings)
        self.go_back.connect(self.setMode)

    def setMode(self):
        if 'net_mode' in self.net_settings:
            mode = self.net_settings['net_mode']
        else:
            mode = 'user'

        if mode == 'default':
            self.defaultMode()
            self.radio_default.setChecked(True)
        elif mode == 'dhcp':
            self.dhcpMode()
            self.radio_dhcp.setChecked(True)
        else:
            self.userMode()
            self.radio_user.setChecked(True)

    def protocolAnswer(self, msg):
        if 'answer' in msg:
            answer = msg['answer']

            if 'settings' in answer and answer['settings'] == 'ok':
                self.net_settings = msg['settings']
            elif 'sysinfo' in answer and answer['sysinfo'] == 'ok':
                self.sysinfo = msg['sysinfo']
                self.setMode()

    def charClick(self):
        text = self.current.text()
        var = self.current.property('var')

        if len(text) < 15:
            x = text + self.sender().text()
            self.current.setText(x)
            self.net_settings[var] = x

    def deleteClick(self):
        text = self.current.text()
        var = self.current.property('var')

        x = text[:-1]
        self.current.setText(x)
        self.net_settings[var] = x

    def selectLine(self):
        if self.current == self.sender():
            self.group.setExclusive(False)
            self.current.setChecked(False)
            self.group.setExclusive(True)
            self.current = None
            self.group3.hide()
        else:
            self.group3.show()
            self.current = self.sender()

    def defaultMode(self):
        self.mode = 'default'
        self.net_ip.setEnabled(False)
        self.net_gateway.setEnabled(False)
        self.net_mask.setEnabled(False)

        self.group.setExclusive(False)
        self.net_ip.setChecked(False)
        self.net_gateway.setChecked(False)
        self.net_mask.setChecked(False)
        self.group.setExclusive(True)

        sysinfo = self.sysinfo

        if 'IP default' in sysinfo:
            self.net_ip.setText(sysinfo['IP default'])
        else:
            self.net_ip.setText('')

        if 'Gateway default' in sysinfo:
            self.net_gateway.setText(sysinfo['Gateway default'])
        else:
            self.net_gateway.setText('')

        if 'Mask default' in sysinfo:
            self.net_mask.setText(sysinfo['Mask default'])
        else:
            self.net_mask.setText('')

        self.group2.show()
        self.group3.hide()
        self.lbl_dhcp.hide()

    def dhcpMode(self):
        self.mode = 'dhcp'
        self.group2.hide()
        self.group3.hide()
        self.lbl_dhcp.show()

    def userMode(self):
        self.mode = 'user'
        self.net_ip.setEnabled(True)
        self.net_gateway.setEnabled(True)
        self.net_mask.setEnabled(True)

        settings = self.net_settings

        if 'net_ip' in settings:
            self.net_ip.setText(settings['net_ip'])
        else:
            self.net_ip.setText('')

        if 'net_gateway' in settings:
            self.net_gateway.setText(settings['net_gateway'])
        else:
            self.net_gateway.setText('')

        if 'net_mask' in settings:
            self.net_mask.setText(settings['net_mask'])
        else:
            self.net_mask.setText('')

        self.group2.show()
        self.lbl_dhcp.hide()

    def show(self):
        self.protocol.getSettings('net')
        self.protocol.getSysinfo()
        super().show()
