from PyQt5.QtWidgets import QWidget, QPushButton, QRadioButton, QButtonGroup

from users.new_card import UsersNewCardWidget
from users.user_window_default import UserWindowDefaultWidget


class UsersCardWidget(UserWindowDefaultWidget):
    def __init__(self, parent=None):
        super().__init__(parent)

        self.protocol.answer.connect(self.protocolAnswer)
        self.protocol.events.connect(self.eventAnswer)

        up = QPushButton(None, self)
        up.setObjectName('up')
        up.setGeometry(476+10, 98, 754-686, 137-85)
        up.clicked.connect(self.upClick)
        self.up = up

        down = QPushButton(None, self)
        down.setObjectName('down')
        down.setGeometry(476+10, 265, 754-686, 449-398)
        down.clicked.connect(self.downClick)
        self.down = down

        button = QPushButton(_("Добавить карту"), self)
        button.setGeometry(50, 382, 331-50, 460-382)
        button.setObjectName('new')
        button.clicked.connect(UsersNewCardWidget(self).show)

        button = QPushButton(_("Удалить карту"), self)
        button.setGeometry(469, 382, 751-469, 460-382)
        button.clicked.connect(self.btn_delete_click)
        button.setObjectName('del')
        self.btn_delete = button

        self.buttons = None
        self.offset = 0

    def show(self):
        super().show()
        self.loadCards()

    def loadCards(self):
        self.btn_delete.hide()
        self.cards = []
        self.offset = 0
        print(
            'trying to get cards for ', self.selected_user['name'],
            ' [', self.selected_user['account_id'], ']')
        self.protocol.getCard(self.selected_user['account_id'])

    def addButtons(self):
        if self.buttons is not None:
            self.buttons.deleteLater()

        if len(self.cards) > 3:
            self.up.show()
            self.down.show()
        else:
            self.up.hide()
            self.down.hide()

        buttons = QWidget(self)
        buttons.setGeometry(256, 89, 476-256, 382-89)
        buttons.setObjectName('buttons')

        buttons.group = QButtonGroup(buttons)
        buttons.group.buttonClicked.connect(self.btn_delete.show)

        y = 0

        for item in self.cards[self.offset:(self.offset+3)]:
            button = QRadioButton(item, buttons)
            buttons.group.addButton(button)
            button.setGeometry(0, y, 266, 48)
            button.show()
            y += 93

        buttons.show()
        self.buttons = buttons

    # Protocol handlers #

    def protocolAnswer(self, msg):
        if 'answer' not in msg:
            return

        answer = msg['answer']

        if 'uid' in answer:
            uid = msg['uid']

            if answer['uid'] == 'ok':
                if 'set' in answer:
                    if answer['set'] == 'own':
                        # and 'user_acct' not in uid:
                        self.loadCards()
                else:
                    self.cards.append(uid['value'])
            elif answer['uid'] == 'done':
                self.addButtons()

    def eventAnswer(self, msg):
        if 'code' not in msg:
            return

        if 'id_type' in msg and msg['id_type'] == 'finger':
            return

        # if msg['code'] in (1, 11, 12, 13):
        #     self.protocol.unlock('ignore')

    # Button handlers #

    def upClick(self):
        if self.offset >= 3:
            self.offset -= 3
            self.addButtons()

    def downClick(self):
        if self.offset < len(self.cards) - 3:
            self.offset += 3
            self.addButtons()

    def btn_delete_click(self):
        if self.buttons.group.checkedButton() is None:
            return
        self.protocol.deleteCard(self.buttons.group.checkedButton().text())
