import base64

from PyQt5.QtWidgets import QLabel, QPushButton, QRadioButton, QButtonGroup
from PyQt5.QtGui import QPixmap

from users.clear_finger import UsersClearFingerWidget

from users.user_window_default import UserWindowDefaultWidget


class UsersFingerWidget(UserWindowDefaultWidget):

    acct = None
    name = None

    def __init__(self, parent=None):
        super().__init__(parent)

        self.go_back.connect(self.protocol.scanOffFinger)

        self.quality = 100

        label = QLabel(_("Качество"), self)
        label.setGeometry(264, 91, 429-264, 155-91)
        label.setObjectName('quality')

        button = QPushButton('<', self)
        button.setGeometry(429, 91, 537-429, 155-91)
        button.setObjectName('left')
        button.clicked.connect(lambda: self._change_quality(-1))

        quality_info = QLabel(None, self)
        quality_info.setGeometry(537, 91, 680-537, 155-91)
        quality_info.setObjectName('quality_info')
        self.quality_info = quality_info

        button = QPushButton('>', self)
        button.setGeometry(680, 91, 786-680, 155-91)
        button.setObjectName('right')
        button.clicked.connect(lambda: self._change_quality(1))

        self._change_quality(0)  # show qualityon screen and set it in limits

        button = QPushButton(_("Добавить отпечаток"), self)
        button.setGeometry(12, 382, 326-12, 460-382)
        button.clicked.connect(self._on_btn_add_click)
        button.setObjectName('new')
        self.btn_add_finger = button

        self.numbers = QButtonGroup(self)
        for num in range(0, 10):
            button = QRadioButton(None, self)
            button.setGeometry(12 + num * 77, 270, 78, 99)
            button.label = QLabel(str(num + 1), button)
            button.label.resize(78, 99)
            self.numbers.addButton(button, num)

        pic = QLabel(self)
        pic.setGeometry(51, 74, 225-51, 254-74)
        pic.setObjectName('pic')
        pic.setScaledContents(True)
        self.pic = pic
        self.pixmap = QPixmap()

        info = QLabel(self)
        info.setGeometry(264, 174, 787-264, 255-174)
        info.setObjectName('info')
        self.info = info

        button = QPushButton(_("Удалить все отпечатки"), self)
        button.setGeometry(473, 382, 783-473, 460-382)
        button.setObjectName('clear')
        page = UsersClearFingerWidget(self)
        page.accept.connect(
            lambda: self.protocol.clearFinger(self.selected_user['account_id']))
        button.clicked.connect(page.show)
        self.btn_delete_all = button

        self.protocol.answer.connect(self._on_protocol_answer)

    def show(self):
        self.btn_add_finger.hide()
        super().show()
        self._enable_all_positions()
        self.protocol.getFinger(self.selected_user['account_id'])

    # # #

    def _change_quality(self, delta):
        self.quality = min(100, max(1, self.quality + delta))
        self.quality_info.setText(str(self.quality))

    # # #

    def _select_first_available(self):
        for button in self.numbers.buttons():
            if button.isEnabled():
                button.setChecked(True)
                self.btn_add_finger.show()
                return
        self.btn_add_finger.hide()

    def _enable_all_positions(self):
        self.btn_delete_all.hide()
        self.pixmap.fill()
        for button in self.numbers.buttons():
            button.setEnabled(True)

    def _disable_fingertip_position(self, num):
        self.numbers.button(num).setEnabled(False)
        self.btn_delete_all.show()

    def _on_all_fingertips_removed(self):
        self.info.setText(_("Удалены все отпечатки"))
        self._enable_all_positions()
        self._select_first_available()

    def _on_fingertip_added(self, num):
        self.info.setStyleSheet('background-color: transparent;')
        self._disable_fingertip_position(num)
        self._select_first_available()

    # # #

    def _on_btn_add_click(self):
        self.info.setText(_("Необходимо предъявить палец 3 раза"))
        self.info.setStyleSheet('background-color: #CA0002;')
        self.protocol.scanOnFinger(
            self.selected_user['account_id'],
            self.numbers.checkedId(),
            self.quality)

    def _on_protocol_answer(self, msg):
        if 'event_finger' in msg:
            event = msg['event_finger']

            if event['event'] == 'image':
                image = base64.b64decode(event['image'])
                self.pixmap.loadFromData(image)
                print(self.pixmap.height(), self.pixmap.width(), len(image))
                self.pic.setPixmap(self.pixmap)
            else:
                self.info.setText(_(event['event']))

            if event['event'] == 'Success add user template':
                self._on_fingertip_added(event['finger_num'])

        elif 'answer' in msg:
            answer = msg['answer']

            if 'finger' not in answer:
                return
            res = answer['finger']

            if res == 'ok':
                cmd = msg['finger']

                if 'user_acct' not in cmd:
                    return
                if cmd['user_acct'] != self.selected_user['account_id']:
                    return
                if 'set' in answer:
                    self._on_all_fingertips_removed()
                else:
                    self._disable_fingertip_position(cmd['finger_num'])
            elif res == 'done':
                self._select_first_available()
